{$APPTYPE CONSOLE}

program Elas2d;

//     Rsolution d'un problme d'lasticit plane
//     par la mthode des lments finis

//     Programme crit en PASCAL - Application console
//     Compilateur DELPHI
//     mai 2009

//     Yves DEBARD
//     Dpartement Gnie Mcanique et Productique
//     Institut Universitaire de Technologie du MANS

uses SysUtils,Windows;

const

  Max_noeuds = 100 ;
  Max_elements = 100 ;
  Max_liaisons = 20 ;
  Max_charges =20 ;
  NDDLE = 6 ;          // nombre de degrs de libert d'un lment
  Max_ddl = 2*Max_noeuds ;

  PB_CONTRAINTES_PLANES = 1 ;
  PB_DEFORMATIONS_PLANES = 2 ;

  ID_DX_NUL = 1 ;
  ID_DY_NUL = 2 ;
  ID_ROTULE = 3 ;

  ID_FORCE_VOLUMIQUE = 1 ;
  ID_FORCE_LINEIQUE = 2 ;
  ID_PESANTEUR = 3 ;

  ACCELERATION_PESANTEUR = 10 ;

type

  TMatrice_elastique = array [1..3,1..3] of Double;
  TMatrice_derivees = array [1..3,1..NDDLE] of Double;
  TVecteur_elementaire = array [1..NDDLE] of Double;
  TMatrice_elementaire = array [1..NDDLE,1..NDDLE] of Double;
  TLocalisation_ddl_elementaires = array [1..NDDLE] of Integer;
  TVecteur_global = array [1..Max_ddl] of Double;
  TMatrice_globale = array [1..Max_ddl,1..Max_ddl] of Double;
  TEtat_deformation_contrainte = record
      exx,eyy,gxy,ezz,sxx,syy,sxy,szz:Double;
    end;

  TNoeud = record
      x,y:Double;
    end;
  TElement = class
      Noeuds: array [1..3] of Integer;
      Epaisseur,Young,Poisson:Double;
      constructor Create;
      function Aire:Double;
      function Volume:Double;
      procedure Matrice_elastique(var d:TMatrice_elastique);
      function Matrice_derivees(var aire:Double;var mat_B:TMatrice_derivees):Boolean;
      function Rigidite_elementaire(var ke:TMatrice_elementaire):Boolean;
      procedure Deformations_contraintes(var def_sig:TEtat_deformation_contrainte);
    end;
  TElements = class
      Nombre:Integer;
      Table:array [1..Max_elements] of TElement;
      constructor Create;
      destructor Destroy;override;
      function Index(e:TElement):Integer;
    end;

  TLiaison = class
      Type_liaison:Byte;
      Noeud:Integer;
      Rx,Ry:Double;
      constructor Create;
    end;
  TLiaisons = class
      Nombre:Integer;
      Table:array [1..Max_liaisons] of TLiaison;
      constructor Create;
      destructor Destroy;override;
    end;

  TCharge = class
      Type_charge:Byte;
      Entites:array [1..2] of Integer;
      Fx,Fy:Double;
      constructor Create;
    end;
  TCharges = class
      Nombre:Integer;
      Table:array [1..Max_charges] of TCharge;
      constructor Create;
      destructor Destroy;override;
    end;

  TStructure = class
      Nom_du_projet:String;
      Probleme,NNoeuds,NLiaisons,NCharges,NDdl:Integer;
      Noeuds:array [1..Max_noeuds] of TNoeud;
      Elements:TElements;
      Liaisons:TLiaisons;
      Charges:TCharges;
      Matkg:TMatrice_globale;
      Vectg:TVecteur_global;
      fres:Text;
      Fichier_resultat:Boolean;
      constructor Create;
      destructor Destroy;override;
      function Lire_les_donnees:Boolean;
      procedure Localisation_ddl_noeud(inoeud:Integer;var ddl_ux,ddl_uy:Integer);
      procedure Localisation_ddl_element(i_element:Integer;
                            var locddl:TLocalisation_ddl_elementaires);
      function Assembler_la_matrice_de_rigidite:Boolean;
      procedure Assembler_le_vecteur_force;
      procedure Limites;
      function Calculer_les_deplacements:Boolean;
      procedure Actions_de_liaison;
      procedure Editions;
      procedure Executer;
    end;

var Structure:TStructure;

//=============================================================================

constructor TElement.Create;
  begin
    inherited Create;
  end;

function TElement.Aire:Double;
  var k,n:Integer;
      x,y:array [1..3] of Double;
  begin
    for k:=1 to 3 do
      begin
        n:=Noeuds[k];
        x[k]:=Structure.Noeuds[n].x;
        y[k]:=Structure.Noeuds[n].y;
      end;
    result:=(x[2]-x[1])*(y[3]-y[1])-(x[3]-x[1])*(y[2]-y[1]);
  end;

function TElement.Volume:Double;
  begin
    result:=Aire*Epaisseur;
  end;

procedure TElement.Matrice_elastique(var d:TMatrice_elastique);
  begin
    FillChar(d,Sizeof(d),#0);
    case Structure.Probleme of
      PB_CONTRAINTES_PLANES :
        begin
          d[1,1]:=Young/(1-Sqr(Poisson));d[1,2]:=Poisson*d[1,1];
          d[2,1]:=d[1,2];d[2,2]:=d[1,1];
          d[3,3]:=Young/(2*(1+Poisson));
        end;
      PB_DEFORMATIONS_PLANES :
        begin
          d[1,1]:=Young*(1-Poisson)/((1+Poisson)*(1-2*Poisson));
          d[1,2]:=Poisson*d[1,1]/(1-Poisson);
          d[2,1]:=d[1,2];d[2,2]:=d[1,1];
          d[3,3]:=Young/(2*(1+Poisson));
        end;
    end;
  end;

function TElement.Matrice_derivees(var aire:Double;var mat_B:TMatrice_derivees):Boolean;
  var xn,yn,a,b:array [1..3] of Double;
      k,n:Integer;
  begin
    result:=False;
    for k:=1 to 3 do
      begin
        n:=Noeuds[k];
        xn[k]:=Structure.Noeuds[n].x;
        yn[k]:=Structure.Noeuds[n].y;
      end;
    a[1]:=yn[2]-yn[3];a[2]:=yn[3]-yn[1];a[3]:=yn[1]-yn[2];
    b[1]:=xn[3]-xn[2];b[2]:=xn[1]-xn[3];b[3]:=xn[2]-xn[1];
    aire:=xn[1]*a[1]+xn[2]*a[2]+xn[3]*a[3];
    if aire<1e-8
      then
        begin
          writeln(Structure.fres,'L''aire de l''lment ',Structure.Elements.Index(Self),' est nulle : ',aire);
          Exit;
        end;
    FillChar(mat_B,Sizeof(mat_B),#0);
    for k:=1 to 3 do
      begin
        a[k]:=a[k]/aire;
        b[k]:=b[k]/aire;
        n:=2*k-1;
        mat_B[1,n]:=a[k];
        mat_B[2,n+1]:=b[k];
        mat_B[3,n]:=b[k];
        mat_B[3,n+1]:=a[k];
      end;
    aire:=0.5*aire;
    result:=True;
  end;

function TElement.Rigidite_elementaire(var ke:TMatrice_elementaire):Boolean;

  var aire,c:Double;
      D:TMatrice_elastique;
      B:TMatrice_derivees;
      i,j,k:Integer;
      temp:array [1..3,1..NDDLE] of Double;

  begin

    result:=False;

    Matrice_elastique(D);
    if not Matrice_derivees(aire,B) then Exit;

    c:=aire*Epaisseur;
    for i:=1 to 3 do
      for j:=1 to 3 do D[i,j]:=D[i,j]*c;

    for i:=1 to 3 do
      for j:=1 to NDDLE do
        begin
          temp[i,j]:=0;
          for k:=1 to 3 do temp[i,j]:=temp[i,j]+D[i,k]*B[k,j];
        end;
    for i:=1 to NDDLE do
      for j:=1 to NDDLE do
        begin
          ke[i,j]:=0;
          for k:=1 to 3 do ke[i,j]:=ke[i,j]+B[k,i]*temp[k,j];
        end;

    result:=True;

  end;

procedure TElement.Deformations_contraintes(var def_sig:TEtat_deformation_contrainte);

  var j,k:Integer;
      depl:TVecteur_elementaire;
      locddl:TLocalisation_ddl_elementaires;
      D:TMatrice_elastique;
      B:TMatrice_derivees;
      x:Double;
      eps,sig:array [1..3] of Double;

  begin

    k:=Structure.Elements.Index(Self);
    Structure.Localisation_ddl_element(k,locddl);
    for k:=1 to NDDLE do depl[k]:=Structure.Vectg[locddl[k]];
    Matrice_elastique(D);
    Matrice_derivees(x,B);
    for j:=1 to 3 do
      begin
        eps[j]:=0;
        for k:=1 to NDDLE do eps[j]:=eps[j]+B[j,k]*depl[k];
      end;
    for j:=1 to 3 do
      begin
        sig[j]:=0;
        for k:=1 to 3 do sig[j]:=sig[j]+D[j,k]*eps[k];
      end;
    with def_sig do
      begin
        exx:=eps[1];eyy:=eps[2];ezz:=0;gxy:=eps[3];
        sxx:=sig[1];syy:=sig[2];szz:=0;sxy:=sig[3];
        if Structure.Probleme = PB_CONTRAINTES_PLANES
          then ezz:=-Poisson*(Sxx+Syy)/Young
          else szz:=Poisson*(Sxx+Syy);
      end;

  end;

//==============================================================================

constructor TElements.Create;
  begin
    inherited Create;
    Nombre:=0;
  end;

destructor TElements.Destroy;
  var i:Integer;
  begin
    for i:=1 to Nombre do Table[i].Free;
    inherited Destroy;
  end;

function TElements.Index(e:TElement):Integer;
  var i:Integer;
  begin
    result:=0;
    if e<>nil
      then
        for i:=1 to Nombre do
          begin
            if Table[i]=e then begin result:=i;Exit;end;
          end;
  end;

//==============================================================================

constructor TLiaison.Create;
  begin
    inherited Create;
  end;

//==============================================================================

constructor TLiaisons.Create;
  begin
    inherited Create;
    Nombre:=0;
  end;

destructor TLiaisons.Destroy;
  var i:Integer;
  begin
    for i:=1 to Nombre do Table[i].Free;
    inherited Destroy;
  end;

//==============================================================================

constructor TCharge.Create;
  begin
    inherited Create;
  end;

//==============================================================================

constructor TCharges.Create;
  begin
    inherited Create;
    Nombre:=0;
  end;

destructor TCharges.Destroy;
  var i:Integer;
  begin
    for i:=1 to Nombre do Table[i].Free;
    inherited Destroy;
  end;

//==============================================================================

constructor TStructure.Create;
  begin
    inherited Create;
    Elements:=TElements.Create;
    Liaisons:=TLiaisons.Create;
    Charges:=TCharges.Create;
  end;

destructor TStructure.Destroy;
  var s:String;
  begin
    Elements.Free;
    Liaisons.Free;
    Charges.Free;
    if Fichier_resultat
      then
        begin
          close(fres);
          s:='notepad '+nom_du_projet+'r.txt';
          Winexec(PChar(s),SW_NORMAL);
        end;
    inherited Destroy;
  end;

function TStructure.Lire_les_donnees:Boolean;

   // les donnes sont lues dans un fichier texte dont l'extension est .txt

   var i,k:Integer;
       nom_fichier,ligne:String;
       f:text;

   function Mot_cle(mc,s:String):Boolean;
     var k:Integer;
         s1,s2:String;
     begin
       s1:='';for k:=1 to Length(mc) do if mc[k]<>' ' then s1:=s1+Upcase(mc[k]);
       s2:='';for k:=1 to Length(s) do if s[k]<>' ' then s2:=s2+Upcase(s[k]);
       s2:=copy(s2,1,Length(s1));
       result:=s1=s2;
     end;

  begin

    result:=False;

     Probleme:=0;
     NNoeuds:=0;
     NLiaisons:=0;
     NCharges:=0;

    writeln('Rappel : LES DONNES SONT LUES DANS UN FICHIER DONT L''EXTENSION EST .TXT');
    writeln;

    write('Nom du fichier ( sans extension ) ? ');
    readln(Nom_du_projet);
    if Nom_du_projet=''
      then begin result:=False;Exit;end;
    nom_fichier:=Nom_du_projet+'.txt';

    assign(f,nom_fichier);{$i-}reset(f);{$i+}
    if Ioresult<>0 then begin result:=False;Exit;end;

    assign(fres,nom_du_projet+'r.txt');{$i-}rewrite(fres);{$i+}
    if Ioresult<>0 then begin close(f);result:=False;Exit;end;
    Fichier_resultat:=True;

     writeln(fres);writeln(fres,'Lecture du fichier : ',nom_fichier);

     repeat
       if Eof(f) then begin Close(f);Exit;end;
       readln(f,ligne);
     until Mot_cle('$deb',ligne);

     repeat

       if Eof(f) then begin Close(f);Exit;end;
       readln(f,ligne);
       writeln(ligne);

       if Mot_cle('$pro',ligne) then readln(f,Probleme);
       if Mot_cle('$noe',ligne)
         then
           begin
             readln(f,NNoeuds);
             for i:=1 to NNoeuds do with Noeuds[i] do readln(f,x,y);
           end;
       if Mot_cle('$ele',ligne)
         then
           begin
             readln(f,Elements.Nombre);
             for i:=1 to Elements.Nombre do
               begin
                 Elements.Table[i]:=TElement.Create;
                 with Elements.Table[i] do
                   begin
                     for k:=1 to 3 do read(f,Noeuds[k]);
                     readln(f,Young,Poisson,Epaisseur);
                  end;
               end;
           end;
       if Mot_cle('$lia',ligne)
         then
           begin
             readln(f,NLiaisons);
             for i:=1 to NLiaisons do
               begin
                 Liaisons.Table[i]:=TLiaison.Create;
                 with Liaisons.Table[i] do readln(f,Type_liaison,Noeud);
               end;
           end;
       if Mot_cle('$cha',ligne)
         then
           begin
             readln(f,NCharges);
             for i:=1 to NCharges do
               begin
                 Charges.Table[i]:=TCharge.Create;
                 with Charges.Table[i] do
                   begin
                     read(f,Type_charge);
                     case Type_charge of
                       ID_FORCE_VOLUMIQUE:read(f,Entites[1],Fx,Fy);
                       ID_FORCE_LINEIQUE:read(f,Entites[1],Entites[2],Fx,Fy);
                     end;
                     readln(f);
                 end;
               end;
           end;

     until Mot_cle('$fin',ligne);

     close(f);

     if (NNoeuds=0) or (Elements.Nombre=0) or (NLiaisons=0) or (NCharges=0)
       then Probleme:=0;

    result:=True;

  end;

procedure TStructure.Localisation_ddl_noeud(inoeud:Integer;var ddl_ux,ddl_uy:Integer);
  begin
    ddl_ux:=2*inoeud-1;
    ddl_uy:=ddl_ux+1;
  end;

procedure TStructure.Localisation_ddl_element(i_element:Integer;
                            var locddl:TLocalisation_ddl_elementaires);
  var element:TElement;
      k:Integer;
  begin
    element:=Elements.Table[i_element];
    for k:=1 to 3 do
      Localisation_ddl_noeud(element.Noeuds[k],locddl[2*k-1],locddl[2*k]);
  end;

function TStructure.Assembler_la_matrice_de_rigidite:Boolean;

   var ielement,ice,ile,icg,ilg:Integer;
       element:TElement;
       ke:TMatrice_elementaire;
       locddl:TLocalisation_ddl_elementaires;

   begin

     writeln(fres,'Assemblage de la matrice de rigidit');

     result:=True;

     NDdl:=2*NNoeuds;
     for ilg:=1 to NDdl do                   
       for icg:=1 to NDdl do Matkg[ilg,icg]:=0;

     for ielement:=1 to Elements.Nombre do
       begin
         element:=Elements.Table[ielement];
         if not element.Rigidite_elementaire(ke) then Exit;
         Localisation_ddl_element(ielement,locddl);
         for ile:=1 to NDDLE do
           for ice:=1 to NDDLE do
             begin
               ilg:=locddl[ile];
               icg:=locddl[ice];
               Matkg[ilg,icg]:=Matkg[ilg,icg]+ke[ile,ice];
             end;
        end;

    result:=True;

    end;

procedure TStructure.Assembler_le_vecteur_force;

   var i,k,ddl_ux,ddl_uy:Integer;
       xn,yn:array [1..2] of Double;
       longueur,volume,fx,fy:Double;
       charge:TCharge;
       element:TElement;
       locddl:TLocalisation_ddl_elementaires;

   begin

     writeln(fres,'Assemblage du vecteur force global');

     for i:=1 to NDdl do Vectg[i]:=0;
     for i:=1 to NCharges do
       begin
         charge:=Charges.Table[i];
         case charge.Type_charge of
           ID_FORCE_VOLUMIQUE:
             begin
               element:=Elements.Table[charge.Entites[1]];
               volume:=element.Volume;
               fx:=volume*charge.Fx;fy:=volume*charge.Fy;
               Localisation_ddl_element(charge.Entites[1],locddl);
               for k:=1 to 3 do
                 begin
                   Vectg[locddl[2*k-1]]:=Vectg[locddl[2*k-1]]+fx;
                   Vectg[locddl[2*k]]:=Vectg[locddl[2*k]]+fy;
                 end;
             end;
           ID_FORCE_LINEIQUE:
             begin
               for k:=1 to 2 do
                 with Noeuds[charge.Entites[k]] do begin xn[k]:=x;yn[k]:=y;end;
               longueur:=Sqrt(Sqr(xn[2]-xn[1])+Sqr(yn[2]-yn[1]));
               fx:=charge.Fx*longueur*0.5;
               fy:=charge.Fy*longueur*0.5;
               for k:=1 to 2 do
                 begin
                   Localisation_ddl_noeud(charge.Entites[k],ddl_ux,ddl_uy);
                   Vectg[ddl_ux]:=Vectg[ddl_ux]+fx;
                   Vectg[ddl_uy]:=Vectg[ddl_uy]+fy;
                 end;
             end;
         end;
       end;

   end;

procedure TStructure.Limites;

   var i,ddl_ux,ddl_uy:Integer;

   procedure Modifier_ligne_colonne(ddl:Integer);
      var j:Integer;
      begin
        for j:=1 to NDdl do Matkg[j,ddl]:=0;
        for j:=1 to NDdl do Matkg[ddl,j]:=0;
        Matkg[ddl,ddl]:=1;Vectg[ddl]:=0;
      end;

   begin

     writeln(fres,'Mise en place des liaisons');
     writeln(fres,'Initialisation des actions de liaison');

     for i:=1 to NLiaisons do
       with Liaisons.Table[i] do
         begin
           Localisation_ddl_noeud(Noeud,ddl_ux,ddl_uy);
           case Type_liaison of
             ID_DX_NUL:
               begin
                 Rx:=-Vectg[ddl_ux];
                 Modifier_ligne_colonne(ddl_ux);
               end;
             ID_DY_NUL:
               begin
                 Ry:=-Vectg[ddl_uy];
                 Modifier_ligne_colonne(ddl_uy);
               end;
             ID_ROTULE:
               begin
                 Rx:=-Vectg[ddl_ux];
                 Ry:=-Vectg[ddl_uy];
                 Modifier_ligne_colonne(ddl_ux);
                 Modifier_ligne_colonne(ddl_uy);
               end;
           end;
         end;

   end;

function TStructure.Calculer_les_deplacements:Boolean;

   // rsolution du systme linaire symtrique par la mthode de Gauss 

   const eps=1e-10;

   var   iligne,k,icol:Integer;
         c,pivot:Double;

   procedure Erreur;
     begin
       write(fres,'Le systme est mal conditionn : ');
       writeln(fres,'pivot ngatif ou nul = ',pivot);
     end;

   begin

     writeln(fres,'Calcul des dplacements inconnus');

     result:=False;

     // triangulation 

     for iligne:=1 to NDdl-1 do
       begin
         pivot:=Matkg[iligne,iligne];
         if pivot<eps then begin Erreur;Exit;end;
         for k:=iligne+1 to NDdl do
           begin
             c:=Matkg[iligne,k]/pivot;
             Vectg[k]:=Vectg[k]-c*Vectg[iligne];
             for icol:=k to NDdl do
               Matkg[k,icol]:=Matkg[k,icol]-c*Matkg[iligne,icol];
           end;
       end;

     // rsolution du systme triangulaire suprieur 

     pivot:=Matkg[NDdl,NDdl];
     if pivot<eps then begin Erreur;Exit;end;
     Vectg[NDdl]:=Vectg[NDdl]/Matkg[NDdl,NDdl];
     for iligne:=NDdl-1 downto 1 do
       begin
         c:=0;
         for icol:=iligne+1 to NDdl do
           c:=c+Matkg[iligne,icol]*Vectg[icol];
         Vectg[iligne]:=(Vectg[iligne]-c)/Matkg[iligne,iligne];
       end;

    result:=True;

   end;

procedure TStructure.Actions_de_liaison;

   var iliaison,ielement,k,n,ddl:Integer;
       liaison:TLiaison;
       element:TElement;
       depl:TVecteur_elementaire;
       ke:TMatrice_elementaire;
       locddl:TLocalisation_ddl_elementaires;

   begin
     writeln(fres,'Calcul des actions de liaison');
     for iliaison:=1 to NLiaisons do
       begin
         liaison:=Liaisons.Table[iliaison];
         for ielement:=1 to Elements.Nombre do
           begin
             element:=Elements.Table[ielement];
             for n:=1 to 3 do
               if element.Noeuds[n]=liaison.Noeud
                 then
                   begin
                     Localisation_ddl_element(ielement,locddl);
                     for k:=1 to NDDLE do depl[k]:=Vectg[locddl[k]];
                     element.Rigidite_elementaire(ke);
                     ddl:=2*n-1;
                     for k:=1 to NDDLE do liaison.Rx:=liaison.Rx+ke[ddl,k]*depl[k];
                     ddl:=ddl+1;
                     for k:=1 to NDDLE do liaison.Ry:=liaison.Ry+ke[ddl,k]*depl[k];
                   end;
           end;
         Liaisons.Table[iliaison]:=liaison;
       end;
   end;

procedure TStructure.Editions;

   var i,k:Integer;
       element:TElement;
       def_sig:TEtat_deformation_contrainte;

   begin

     writeln(fres);

     write(fres,'Problme : ');
     if Probleme=PB_CONTRAINTES_PLANES
       then writeln(fres,'contraintes planes')
       else writeln(fres,'dformations planes');
     writeln(fres);

     writeln(fres,'Noeuds [ mm ] :');writeln(fres);
     for i:=1 to NNoeuds do
       with Noeuds[i] do writeln(fres,i:3,'  x = ',x:8:3,'  y = ',y:8:3);
     writeln(fres);

     writeln(fres,'lments :');writeln(fres);
     for i:=1 to Elements.Nombre do
       with Elements.Table[i] do
         begin
           write(fres,i:4,'  Noeuds = ( ');
           for k:=1 to 3 do write(fres,Noeuds[k],' ');
           writeln(fres,')  E = ',Young:6:0,' MPa  Nu = ',Poisson:4:2,'  paisseur = ',Epaisseur:5:2,' mm');
         end;
     writeln(fres);

     writeln(fres,'Liaisons :');writeln(fres);
     for i:=1 to NLiaisons do
       with Liaisons.Table[i] do
         begin
           write(fres,'noeud ',Noeud:2);
           case type_liaison of
             ID_DX_NUL:writeln(fres,'   u = 0');
             ID_DY_NUL:writeln(fres,'   v = 0');
             ID_ROTULE:writeln(fres,'   Rotule');
           end;
         end;
     writeln(fres);

     writeln(fres,'Charges :');writeln(fres);
     for i:=1 to NCharges do
       with Charges.Table[i] do
         case Type_charge of
           ID_FORCE_VOLUMIQUE:
             begin
               write(fres,'lment',Entites[1]:3);
               writeln(fres,' : Fx = ',Fx:6:2,'   Fy = ',Fy:6:2,' N/mm3');
             end;
           ID_FORCE_LINEIQUE:
             begin
               write(fres,'noeuds ( ',Entites[1],' ',Entites[2],' ) ');
               writeln(fres,' fx = ',Fx:6:2,'   fy = ',Fy:6:2,' N/mm');
             end;
         end;
     writeln(fres);

     writeln(fres,'Dplacements nodaux [ mm ]:');writeln(fres);
     writeln(fres,'Noeud','Ux':8,'Uy':9);writeln(fres);
     for i:=1 to NNoeuds do
       writeln(fres,i:4,' ',Vectg[2*i-1]:10:6,' ',Vectg[2*i]:10:6);
     writeln(fres);

     writeln(fres,'Contraintes [ MPa ] et dformations :');writeln(fres);
     write(fres,'lement','sxx':5,'syy':8,'sxy':8,'szz':8);
     writeln(fres,'exx':8,'eyy':10,'gxy':10,'ezz':10);
     writeln(fres);
     for i:=1 to Elements.Nombre do
       begin
         element:=Structure.Elements.Table[i];
         element.Deformations_contraintes(def_sig);
         with def_sig do
           begin
             write(fres,i:4,sxx:9:2,syy:8:2,sxy:8:2,szz:8:2);
             writeln(fres,exx:10:6,eyy:10:6,gxy:10:6,ezz:10:6);
           end;
       end;

     writeln(fres);
     writeln(fres,'Actions de liaison [ N ]:');writeln(fres);
     for i:=1 to NLiaisons do
       with Liaisons.Table[i] do
         begin
           write(fres,'noeud ',Noeud,' : ');
           case Type_liaison of
             ID_DX_NUL:
               writeln(fres,'Rx = ',Rx:5:0);
             ID_DY_NUL:
               writeln(fres,'Ry = ',Ry:5:0);
             ID_ROTULE:
               writeln(fres,'Rx = ',Rx:5:0,'  Ry = ',Ry:5:0);
           end;
         end;
     writeln(fres);

   end;

procedure TStructure.Executer;

  begin
    if not Lire_les_donnees then Exit;
    if Probleme=0 then Exit;
    if not Assembler_la_matrice_de_rigidite then Exit;
    Assembler_le_vecteur_force;
    Limites;
    if not Calculer_les_deplacements then Exit;
    Actions_de_liaison;
    Editions;
  end;

//-------   programme principal  ------------

  begin

    writeln;
    writeln('  Rsolution d''un problme d''lasticit plane');
    writeln('      par la mthode des lments finis');
    writeln;
    writeln(' Yves DEBARD - IUT du MANS - mai 2009');
    writeln;

    Structure:=TStructure.Create;
    Structure.Executer;
    Structure.Free;

  end.
